﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Samples.ServiceHosting.StorageClient;
using System.Data.Services.Client;

namespace DataModels
{
    public class User : TableStorageEntity
    {
        // partition key is WLID
        // row key is empty
        public int AgentIdentificationNumber { get; set; }
        public User() { }
        public User(string id, int ain) : base(id, string.Empty)
        {
            AgentIdentificationNumber = ain;
        }
    }

    public class Agent : TableStorageEntity
    {
        // partition key is Agent Identification Number
        // row key is empty string
        public string Name { get; set; }
        public string EmailAddress { get; set; }
        public Agent() { }
        public Agent(int ain, string name, string emailAddress) : base(ain.ToString(), string.Empty)
        {
            this.Name = name;
            this.EmailAddress = emailAddress;
        }
    }

    public class Call : TableStorageEntity
    {
        // partition key is Agent Identification Number
        // row key is an encoded timestamp
        public string Caller { get; set; }
        public DateTime CallTime { get; set; }
        public Call() { }
        public Call(int ain, string caller, DateTime callTime) : base(ain.ToString(), string.Format("{0:d19}", DateTime.MaxValue.Ticks - callTime.Ticks))
        {
            this.Caller = caller;
            this.CallTime = callTime;
        }
    }

    public class CIAContext : TableStorageDataServiceContext
    {
        public DataServiceQuery<User> UserTable { get { return CreateQuery<User>("UserTable"); } }
        public DataServiceQuery<Agent> AgentTable { get { return CreateQuery<Agent>("AgentTable"); } }
        public DataServiceQuery<Call> CallTable { get { return CreateQuery<Call>("CallTable"); } }
    }
}
