using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Mvc.Ajax;
using DataModels;
using Microsoft.Samples.ServiceHosting.StorageClient;
using System.Net;
using WindowsLive;
using Microsoft.ServiceHosting.ServiceRuntime;

namespace TheCIAPickup_WebRole.Controllers
{
    public class HomeController : Controller
    {
        private User TryGetUser(string id)
        {
            return new CIAContext().UserTable.Where(u => u.PartitionKey == id).FirstOrDefault();
        }

        private string GetId(string token)
        {
            return new WindowsLiveLogin(RoleManager.GetConfigurationSetting("wlappid"), RoleManager.GetConfigurationSetting("wlsecret")).ProcessToken(token).Id;
        }

        string LoadCookieData()
        {
            var cookie = Request.Cookies["webauthtoken"];
            ViewData["loggedIn"] = cookie != null;
            string id = null;
            if (cookie != null)
            {
                id = GetId(cookie.Value);
                var user = TryGetUser(id);
                ViewData["ain"] = user != null ? (int?)user.AgentIdentificationNumber : null;
                ViewData["name"] = user != null ? new CIAContext().AgentTable.Where(a => a.PartitionKey == user.AgentIdentificationNumber.ToString()).Single().Name : null;
            }
            else
            {
                id = null;
                ViewData["ain"] = null;
            }
            return id;
        }

        public ActionResult Index()
        {
            LoadCookieData();
            ViewData["phonenumber"] = RoleManager.GetConfigurationSetting("PhoneNumber");
            return View();
        }

        public ActionResult Delete()
        {
            if (LoadCookieData() == null)
            {
                return Content("ERROR: Please log in first.");
            }
            return View();
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Destroy()
        {
            string id = LoadCookieData();
            if (id == null)
            {
                return Content("ERROR: Please log in first.");
            }
            var context = new CIAContext();
            var user = context.UserTable.Where(u => u.PartitionKey == id).FirstOrDefault();
            if (user == null)
            {
                return Content("ERROR: User not found.");
            }
            context.DeleteObject(user);
            foreach (var agent in context.AgentTable.Where(a => a.PartitionKey == user.AgentIdentificationNumber.ToString()))
            {
                context.DeleteObject(agent);
            }
            foreach (var call in context.CallTable.Where(c => c.PartitionKey == user.AgentIdentificationNumber.ToString()))
            {
                context.DeleteObject(call);
            }
            context.SaveChanges();
            return RedirectToAction("index");
        }

        public ActionResult Link()
        {
            if (LoadCookieData() == null)
            {
                return Content("ERROR: Please log in first.");
            }
            return View();
        }

        public ActionResult Calls()
        {
            string id = LoadCookieData();
            if (id == null)
            {
                return Content("ERROR: Please log in first.");
            }
            var user = new CIAContext().UserTable.Where(u => u.PartitionKey == id).FirstOrDefault();
            if (user == null)
            {
                return Content("ERROR: Please sign up for an Agent Identification Number first.");
            }
            var calls = new CIAContext().CallTable.Where(c => c.PartitionKey == user.AgentIdentificationNumber.ToString());
            return View(calls);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult CreateLink(string ain, string emailAddress)
        {
            string id = LoadCookieData();
            if (id == null)
            {
                return Content("ERROR: Please log in first.");
            }
            var context = new CIAContext();
            if (context.UserTable.Where(u => u.PartitionKey == id).FirstOrDefault() != null)
            {
                return Content("ERROR: You're already associated with an Agent Identification Number.  Log out and log in as a different user to use a different AIN.");
            }
            var agent = context.AgentTable.Where(a => a.PartitionKey == ain).FirstOrDefault();
            if (agent == null)
            {
                return Content("ERROR: Invalid Agent Identification Number.");
            }
            if (agent.EmailAddress.ToLower() != emailAddress)
            {
                return Content("ERROR: Email address doesn't match.  Imposter!");
            }
            context.AddObject("UserTable", new User(id, int.Parse(ain)));
            context.SaveChanges();
            return RedirectToAction("index");
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Create(string ain, string name, string emailAddress)
        {
            var context = new CIAContext();
            int ainNumber;
            string id = LoadCookieData();
            if (id == null)
            {
                return Content("ERROR: Please log in first.");
            }
            if (!int.TryParse(ain, out ainNumber) || ainNumber < 0 || ainNumber > 99999)
            {
                return Content("ERROR: Invalid Agent Identification Number.  It should be a five digit number.");
            }
            context.AddObject("AgentTable", new Agent(ainNumber, name, emailAddress));
            try { context.SaveChanges(); }
            catch (Exception e)
            {
                HttpStatusCode status;
                StorageExtendedErrorInformation extendedErrorInfo;
                if (TableStorageHelpers.EvaluateException(e, out status, out extendedErrorInfo) && extendedErrorInfo.ErrorCode == TableErrorCodeStrings.EntityAlreadyExists)
                {
                    return Content("ERROR: That Agent Identification Number is already in use.  Please go back and try another one.");
                }
            }
            context.AddObject("UserTable", new User(id, ainNumber));
            context.SaveChanges();
            return RedirectToAction("index");
        }
    }
}