﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Linq;
using System.Text;
using Microsoft.ServiceHosting.ServiceRuntime;
using Microsoft.Samples.ServiceHosting.StorageClient;
using DataModels;
using System.Net.Mail;
using System.Net;

namespace TheCIAPickup_Worker
{
    public class WorkerRole : RoleEntryPoint
    {
        public override void Start()
        {
            var q = QueueStorage.Create(StorageAccountInfo.GetDefaultQueueStorageAccountFromConfiguration()).GetQueue("callqueue");
            while (true)
            {
                var msg = q.GetMessage(60);
                if (msg != null)
                {
                    var split = msg.ContentAsString().Split(new char[] { '/' }, 3);
                    int ain = int.Parse(split[0]);
                    string caller = split[1];
                    DateTime callTime = new DateTime(long.Parse(split[2]));
                    var call = new Call(ain, caller, callTime);
                    var context = new CIAContext();
                    context.AddObject("CallTable", call);
                    try
                    {
                        context.SaveChanges();
                    }
                    catch (Exception e)
                    {
                        HttpStatusCode status;
                        StorageExtendedErrorInformation extendedErrorInfo;
                        if (!TableStorageHelpers.EvaluateException(e, out status, out extendedErrorInfo) || extendedErrorInfo.ErrorCode != TableErrorCodeStrings.EntityAlreadyExists)
                        {
                            throw;
                        }
                    }
                    var agent = context.AgentTable.Where(a => a.PartitionKey == ain.ToString() && a.RowKey == string.Empty).Single();
                    (new SmtpClient(RoleManager.GetConfigurationSetting("SmtpServer"), int.Parse(RoleManager.GetConfigurationSetting("SmtpPort")))
                    {
                        Credentials = new System.Net.NetworkCredential(RoleManager.GetConfigurationSetting("SmtpUsername"), RoleManager.GetConfigurationSetting("SmtpPassword")),
                        EnableSsl = true
                    }).Send(RoleManager.GetConfigurationSetting("FromAddress"), agent.EmailAddress, string.Format("Received a CIA call from {0}", call.Caller), string.Format("Agent {0},\n\nYou received a phone call via your CIA Agent Identification Number from the phone number \"{1}\" at {2} GMT.", agent.Name, call.Caller, call.CallTime.ToString()));
                    q.DeleteMessage(msg);
                }
                else
                {
                    Thread.Sleep(1000);
                }
            }
        }

        public override RoleStatus GetHealthStatus()
        {
            return RoleStatus.Healthy;
        }
    }
}